@@ -1,15 +1,24 @@
 // revisions: edition2018 edition2021
-// [edition2018] edition:2018
-// [edition2021] edition:2021
+//[edition2018] edition:2018
+//[edition2021] edition:2021
 // run-rustfix
+
 #![warn(clippy::manual_assert)]
+#![allow(dead_code, unused_doc_comments)]
+#![allow(clippy::nonminimal_bool, clippy::uninlined_format_args)]
+
+macro_rules! one {
+    () => {
+        1
+    };
+}
 
 fn main() {
     let a = vec![1, 2, 3];
     let c = Some(2);
     if !a.is_empty()
         && a.len() == 3
-        && c != None
+        && c.is_some()
         && !a.is_empty()
         && a.len() == 3
         && !a.is_empty()
@@ -54,4 +63,19 @@ fn main() {
     if a.is_empty() || !b.is_empty() {
         panic!("panic5");
     }
+    if a.is_empty() {
+        panic!("with expansion {}", one!())
+    }
+}
+
+fn issue7730(a: u8) {
+    // Suggestion should preserve comment
+    if a > 2 {
+        // comment
+        /* this is a
+        multiline
+        comment */
+        /// Doc comment
+        panic!("panic with comment") // comment after `panic!`
+    }
 }
